@@ -3,10 +3,12 @@
 from .gaps_identification import (
     gaps_identification_iqr,
     gaps_identification_modified_z_scores,
+    gaps_identification_threshold,
     gaps_identification_z_scores,
 )
 from .negative_running_hours import negative_running_hours_check
 from .outliers import extreme
+from .uncertainty import uncertainty_rstd
 
 
 TOOLBOX_NAME = "Data quality"
@@ -15,6 +17,8 @@ __all__ = [
     "gaps_identification_z_scores",
     "gaps_identification_modified_z_scores",
     "gaps_identification_iqr",
+    "gaps_identification_threshold",
     "extreme",
     "negative_running_hours_check",
+    "uncertainty_rstd",
 ]
