@@ -4,6 +4,7 @@ import pandas as pd
 from statsmodels.tsa.arima.model import ARIMA
 
 from ..type_check import check_types
+from ..validations import validate_series_is_not_empty
 
 
 @check_types
@@ -28,9 +29,7 @@ def arma(data: pd.Series, ar_order: int = 2, ma_order: int = 2):
 
     # TODO: Refactor to accepet time window
 
-    # Check length of data
-    if len(data) < 1:
-        raise RuntimeError("No data passed to algorithm.")
+    validate_series_is_not_empty(data)
 
     # Train model on entire dataset and return fit on dataset
     model = ARIMA(data, order=(ar_order, 0, ma_order))
