@@ -4,8 +4,9 @@ import pandas as pd
 
 from numba import jit
 
-from indsl.ts_utils.ts_utils import time_parse
-from indsl.type_check import check_types
+from ..ts_utils.ts_utils import time_parse
+from ..type_check import check_types
+from ..validations import validate_series_is_not_empty
 
 
 @check_types
@@ -50,6 +51,8 @@ def ewma(
     Returns:
         pandas.Series: Smoothed time series.
     """
+    validate_series_is_not_empty(data)
+
     time_window = time_parse(time_window)
     win_pts_lst = np.array(data.rolling(time_window).count().values)
     ewma_vals = np.zeros_like(data)
