@@ -333,3 +333,8 @@ def fill_gaps(
     data[mask] = data.interpolate(method="linear", limit_direction="forward", limit=inter_gap)
 
     return data.fillna(method=method)
+
+
+def number_of_events(out: pd.Series):
+    """Calculate the number of events detected from a step time series."""
+    return len(np.where(out.values == 1)[0]) / 2
