@@ -1,5 +1,6 @@
 # Copyright 2021 Cognite AS
 
+from datetime import datetime, timedelta
 from typing import List, Literal, Optional
 
 import numpy as np
@@ -25,7 +26,7 @@ def round(x, decimals: int):
         decimals (int): number of decimals
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
     """
     return np.round_(x, decimals=decimals)
 
@@ -38,7 +39,7 @@ def floor(x):
         x: time series
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
     """
     return np.floor(x)
 
@@ -51,7 +52,7 @@ def ceil(x):
         x: time series
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
     """
     return np.ceil(x)
 
@@ -64,7 +65,7 @@ def sign(x):
         x: time series
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
     """
     return np.sign(x)
 
@@ -76,14 +77,14 @@ def clip(x, low: float = -np.inf, high: float = np.inf):
     Given an interval, values of the time series outside the interval are clipped to the interval edges.
 
     Args:
-        x (pandas.Series): time series
+        x (pd.Series): time series
         low (float, Optional): Lower limit
             Lower clipping limit. Default: -infinity
         high (float, Optional): Upper limit
             Upper clipping limit. Default: +infinity
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
     """
     return np.clip(x, low, high)
 
@@ -100,7 +101,7 @@ def maximum(x1, x2, align_timesteps: bool = False):
           Automatically align time stamp  of input time series. Default is False.
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
     """
     x1, x2 = auto_align([x1, x2], align_timesteps)
     return np.maximum(x1, x2)
@@ -118,7 +119,7 @@ def minimum(x1, x2, align_timesteps: bool = False):
           Automatically align time stamp  of input time series. Default is False.
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
     """
     x1, x2 = auto_align([x1, x2], align_timesteps)
     return np.minimum(x1, x2)
@@ -135,7 +136,7 @@ def union(series1: pd.Series, series2: pd.Series) -> pd.Series:
         series2 (pd.Series): time series
 
     Returns:
-        pandas.Series: time series
+        pd.Series: time series
 
     Raises:
         UserTypeError: series1 or series2 is not a time series
@@ -309,7 +310,7 @@ def threshold(series: pd.Series, low: float = -np.inf, high: float = np.inf):
     is 1.0 if the input is between the (inclusive) limits, and 0.0 otherwise.
 
     Args:
-        series (pandas.Series): Time series
+        series (pd.Series): Time series
         low (float, Optional): Lower limit
            threshold. Default: -infinity
         high (float, Optional): Upper limit
@@ -323,3 +324,83 @@ def threshold(series: pd.Series, low: float = -np.inf, high: float = np.inf):
         UserTypeError: low or high are not floats
     """
     return series.between(low, high).astype(np.int64)
+
+
+def generate_step_series(flag: pd.Series) -> pd.Series:
+    """Step-wise time series
+    Construct a step-wise time series (with 0-1 values) from a flag time series.
+
+    Args:
+        flag (pd.Series): Binary time series
+
+    Returns:
+        pd.Series: Time series
+    """
+    # add first point as the start of the step time series
+    step_time_series = pd.Series([flag[0]], index=[flag.index[0]])
+
+    # compute actual_value - previous_value
+    flag_shift = flag.diff(1)
+
+    # find end points of 1-0 (decreasng) or 0-1 (increasing) patterns
+    end_point_increasing = flag_shift[flag_shift == 1]
+    end_point_decreasing = flag_shift[flag_shift == -1]
+
+    # add "1" value for 0-1 (increasing) pattern, add "0" value for 1-0 (decreasing) pattern
+    end_point_decreasing_values = pd.Series(
+        [0] * len(end_point_decreasing), index=end_point_decreasing.index, dtype=np.int64
+    )
+    end_point_increasing_values = pd.Series(
+        [1] * len(end_point_increasing), index=end_point_increasing.index, dtype=np.int64
+    )
+
+    # add extra "corner" points
+    extra_point_decreasing_values = pd.Series(
+        [1] * len(end_point_decreasing), index=end_point_decreasing.index - pd.Timedelta(1, unit="ms"), dtype=np.int64
+    )
+    extra_point_increasing_values = pd.Series(
+        [0] * len(end_point_increasing), index=end_point_increasing.index - pd.Timedelta(1, unit="ms"), dtype=np.int64
+    )
+
+    step_time_series = pd.concat(
+        [
+            step_time_series,
+            end_point_increasing_values,
+            end_point_decreasing_values,
+            extra_point_increasing_values,
+            extra_point_decreasing_values,
+        ]
+    )
+    step_time_series = step_time_series.sort_index()
+
+    # add last point if it doesn't exist
+    if flag.index[-1] != step_time_series.index[-1]:
+        step_time_series = pd.concat([step_time_series, pd.Series(flag[-1], index=[flag.index[-1]])])
+
+    # remove the last point if the number of 1 values is odd
+    if len(step_time_series[step_time_series == 1]) % 2 != 0:
+        step_time_series = step_time_series[:-1]
+
+    return step_time_series
+
+
+def create_series_from_timesteps(timesteps: List[timedelta]) -> pd.Series:
+    """Time series from timestamps
+
+    Create a time series which starts on the 2021-1-1 and contains data points
+    with the provided timesteps in seconds. All values of the output series are zero.
+
+    Args:
+        timesteps (List[timedelta]): Time steps between points.
+
+    Returns:
+        pd.Series: Time series
+
+    """
+
+    current = datetime(2021, 1, 1)
+    timestamps = [current]
+    for step in timesteps:
+        timestamps.append(timestamps[-1] + step)
+    timestamps = pd.Series(timestamps)
+    return pd.Series(np.zeros(len(timestamps)), index=timestamps)
